import { defu } from "defu";
import { GqlClient } from "ohmygql";
import { ref, useCookie, defineNuxtPlugin, useRuntimeConfig, useRequestHeaders } from "#imports";
export default defineNuxtPlugin((nuxtApp) => {
  if (!nuxtApp?._gqlState) {
    nuxtApp._gqlState = ref({});
    const config = useRuntimeConfig();
    const { clients } = defu(config?.["graphql-client"], config?.public?.["graphql-client"]);
    const cookie = process.server && useRequestHeaders(["cookie"])?.cookie || void 0;
    for (const [name, v] of Object.entries(clients || {})) {
      const host = process.client && v?.clientHost || v.host;
      const proxyCookie = v?.proxyCookies && !!cookie;
      const serverHeaders = process.server && (typeof v?.headers?.serverOnly === "object" && v?.headers?.serverOnly) || void 0;
      if (v?.headers?.serverOnly) {
        delete v.headers.serverOnly;
      }
      const opts = {
        ...(proxyCookie || v?.token?.value || v?.headers) && {
          headers: {
            ...v?.headers && { ...v.headers, ...serverHeaders },
            ...proxyCookie && { cookie }
          }
        }
      };
      const authInit = async (reqOpts) => {
        const token = ref();
        await nuxtApp.callHook("gql:auth:init", { token, client: name });
        if (!token.value) {
          token.value = reqOpts?.token?.value;
        }
        if (token.value === void 0 && typeof v.tokenStorage === "object") {
          if (v.tokenStorage?.mode === "cookie") {
            if (process.client) {
              token.value = useCookie(v.tokenStorage.name).value;
            } else if (cookie) {
              const cookieName = `${v.tokenStorage.name}=`;
              token.value = cookie.split(";").find((c) => c.trim().startsWith(cookieName))?.split("=")?.[1];
            }
          } else if (process.client && v.tokenStorage?.mode === "localStorage") {
            const storedToken = localStorage.getItem(v.tokenStorage.name);
            if (storedToken) {
              token.value = storedToken;
            }
          }
        }
        if (token.value === void 0) {
          token.value = v?.token?.value;
        }
        if (token.value) {
          token.value = token.value.trim();
          const tokenName = token.value === reqOpts?.token?.value ? reqOpts?.token?.name || v?.token?.name : v?.token?.name;
          const tokenType = token.value === reqOpts?.token?.value ? reqOpts?.token?.type === null ? null : reqOpts?.token?.type || v?.token?.type : v?.token?.type;
          const authScheme = !!token.value?.match(/^[a-zA-Z]+\s/)?.[0];
          reqOpts.headers[tokenName] = authScheme ? token.value : !tokenType ? token.value : `${tokenType} ${token.value}`;
          return { name: tokenName, token: reqOpts.headers[tokenName] };
        }
        return void 0;
      };
      nuxtApp._gqlState.value[name] = {
        options: opts,
        instance: GqlClient({
          host,
          useGETForQueries: v?.preferGETQueries,
          middleware: {
            onRequest: async (ctx) => {
              const reqOpts = defu(nuxtApp._gqlState.value?.[name]?.options || {}, { headers: {} });
              await authInit(reqOpts);
              if (reqOpts?.token) {
                delete reqOpts.token;
              }
              ctx.options = defu(ctx.options, reqOpts);
            }
          },
          wsOptions: {
            connectionParams: async () => {
              const reqOpts = defu(nuxtApp._gqlState.value?.[name]?.options || {}, { headers: {} });
              const token = await authInit(reqOpts);
              if (!token) {
                return;
              }
              return { [token.name]: token.token };
            }
          }
        })
      };
    }
  }
});
